/********************************** (C) COPYRIGHT *******************************
 * File Name          : main.c
 * Author             : WCH
 * Version            : V1.0.0
 * Date               : 2020/04/30
 * Description        : Main program body.
 * Copyright (c) 2021 Nanjing Qinheng Microelectronics Co., Ltd.
 * SPDX-License-Identifier: Apache-2.0
 *******************************************************************************/

/*
本例程演示通过PWM—DMA方式控制WS2812B模块：WS2812B渐变色流水算法

根据WS2812B的datasheet，这个灯组的控制方式是单IO控制，以高电平的时间不同来判定当前的数据是“1”还是“0”。
0 code对应一个高电平400ns，低电平850ns的脉冲
1 code对应一个高电平800ns，低电平450ns的脉冲
无论0 code还是1 code的脉冲周期都是1250ns，即1.25us,这1.25us即PWM脉冲周期，频率可设置为800K
reset电平是一个大于50us的低电平表示reset。

当PWM设置的有效电平为高时：
1码：占空比为66%，也就是2/3周期
0码：占空比为33%，也就是1/3周期

显示1个LED灯珠的数据需要24位，GRB3种颜色分别占8位，假设现在有3个灯珠，每个灯需要显示红色也就是R=0xFF需要发送如下的数据：
RESET电平 -> LED1(G(00),R(FF),B(00)) -> LED2(G(00),R(FF),B(00)) -> LED3(G(00),R(FF),B(00))


*/

#include "debug.h"
#include "pwm.h"
#include "stdio.h"
#include "string.h"
#include "stdlib.h"

/* Global typedef */

/* Global define */

/* Global Variable */

uint32_t *RGB;  //颜色列表数组

uint32_t set_Color_Loop(uint8_t color_length);
void out_RGB(uint16_t data_Max_Length,uint16_t colorwidth);
/*********************************************************************
 * @fn      main
 *
 * @brief   Main program.
 *
 * @return  none
 */
int main(void)
{
    NVIC_PriorityGroupConfig(NVIC_PriorityGroup_2);
    Delay_Init();
    USART_Printf_Init(115200);

    printf("SystemClk:%d\r\n", SystemCoreClock);
    printf("This is PWM—WS2812B example\r\n");

    WS2812B_PWM_Init();

    uint32_t len = set_Color_Loop(32);  //每两种颜色之间的位数

    printf("len:%d\r\n", len);

    while(1)
    {
        out_RGB(len,8);
        Delay_Ms(10);
    }
}

/*
colorwidth：每次获取多少位
*/

void out_RGB(uint16_t data_Max_Length,uint16_t colorwidth)
{
    static uint32_t rgb_position = 0;
    for(uint32_t i = 0;i < colorwidth; i++)
    {
        uint16_t c = (rgb_position + i) % data_Max_Length;//末尾颜色数据结束时将首位的数据填充到数组
        send_Data(RGB[c]);
    }
    rgb_position ++;
    rgb_position %= data_Max_Length;
}

/*
 * color_length：每种颜色渐变最大长度，总颜色长度：color_length * 3
 * return 颜色数据总长度
 */
uint32_t set_Color_Loop(uint8_t color_length)
{
    RGB = (uint32_t*)malloc(color_length*3*sizeof(uint32_t)); //分配数组大小
    color_length -= 1;
    for(uint8_t i=0;i<=color_length;i++)
    {
        RGB[i] = (((0xff/color_length)*i)<<8)|((0xff/color_length)*(color_length-i));                       //蓝到绿
        RGB[color_length + 1 +i] = (((0xff/color_length)*i)<<16)|((0xff/color_length)*(color_length-i))<<8; //绿到红
        RGB[((color_length+1)*2) +i] = ((0xff/color_length)*i)|((0xff/color_length)*(color_length-i))<<16;  //红到蓝
    }
    return color_length*3;
}

