#include "pwm.h"

uint16_t LED_BYTE_Buffer[24];   //发送数组缓存
uint32_t LED_WORD_Buffer[8];

void WS2812B_PWM_Init(void)
{
    GPIO_InitTypeDef         GPIO_InitStructure={0};
    TIM_TimeBaseInitTypeDef  TIM_TimeBaseStructure={0};
    TIM_OCInitTypeDef        TIM_OCInitStructure={0};
    DMA_InitTypeDef          DMA_InitStructure={0};

    //使能GPIO、TIM1、DMA时钟
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA|RCC_APB2Periph_TIM1, ENABLE);
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA1, ENABLE);

    //TIM1_CH1 GPIO配置
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_8;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(GPIOA, &GPIO_InitStructure);

    //TIM1初始化配置，PWM频率800K
    TIM_TimeBaseStructure.TIM_Period = 89;
    TIM_TimeBaseStructure.TIM_Prescaler = 0;
    TIM_TimeBaseStructure.TIM_ClockDivision = 0;
    TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;
    TIM_TimeBaseStructure.TIM_RepetitionCounter = 0;
    TIM_TimeBaseInit(TIM1, &TIM_TimeBaseStructure);

    //PWM1模式配置
    TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_PWM1;
    TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;
    TIM_OCInitStructure.TIM_Pulse = 0;
    TIM_OCInitStructure.TIM_OCPolarity = TIM_OCPolarity_High;
    TIM_OC1Init(TIM1, &TIM_OCInitStructure);

    DMA_DeInit(DMA1_Channel2);
    DMA_InitStructure.DMA_PeripheralBaseAddr = (uint32_t)&TIM1->CH1CVR;
    DMA_InitStructure.DMA_MemoryBaseAddr = (uint32_t)LED_BYTE_Buffer;
    DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralDST;
    DMA_InitStructure.DMA_BufferSize = 24;
    DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
    DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Enable;
    DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_HalfWord;
    DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_HalfWord;
    DMA_InitStructure.DMA_Mode = DMA_Mode_Normal;
    DMA_InitStructure.DMA_Priority = DMA_Priority_High;
    DMA_InitStructure.DMA_M2M = DMA_M2M_Disable;

    DMA_Init(DMA1_Channel2, &DMA_InitStructure);

    TIM_CtrlPWMOutputs(TIM1, ENABLE);
    TIM_OC1PreloadConfig(TIM1, TIM_OCPreload_Enable);
    TIM_DMACmd(TIM1, TIM_DMA_CC1, ENABLE);
}


void send_Data(uint32_t rgb)
{

    uint8_t r = (rgb&0xff0000)>>16;
    uint8_t g = (rgb&0x00ff00)>>8;
    uint8_t b = (rgb&0xff);
    for(uint16_t i=0;i<8;i++)
    {
        LED_BYTE_Buffer[i] = (0x80&g)>0?TIMING_ONE:TIMING_ZERO;g <<= 1;
    }
    for(uint16_t i=0;i<8;i++)
    {
        LED_BYTE_Buffer[8 + i] = (0x80&r)>0?TIMING_ONE:TIMING_ZERO;r <<= 1;
    }
    for(uint16_t i=0;i<8;i++)
    {
        LED_BYTE_Buffer[16 + i] = (0x80&b)>0?TIMING_ONE:TIMING_ZERO;b <<= 1;
    }
    DMA_SetCurrDataCounter(DMA1_Channel2, 24);  // load number of bytes to be transferred
    DMA_Cmd(DMA1_Channel2, ENABLE);             // enable DMA channel 2
    TIM_Cmd(TIM1, ENABLE);                      // enable Timer 1
    while(!DMA_GetFlagStatus(DMA1_FLAG_TC2)) ;  // wait until transfer complete
    TIM_Cmd(TIM1, DISABLE);                     // disable Timer 1
    DMA_Cmd(DMA1_Channel2, DISABLE);            // disable DMA channel 2
    DMA_ClearFlag(DMA1_FLAG_TC2);               // clear DMA1 Channel 2 transfer complete flag
}

